@@ -164,13 +164,6 @@ func (row *Row) AddLine(line *GraphRowLine) {
 		if line.containsRune('~') {
 			line.Flags = Elided
 		} else {
-			if row.Commit.CommitId == "" {
-				commitIdIdx := line.FindPossibleCommitIdIdx(0)
-				if commitIdIdx != -1 {
-					row.Commit.CommitId = line.Segments[commitIdIdx].Text
-					line.Flags = Revision | Highlightable
-				}
-			}
 			lastLine := row.Lines[len(row.Lines)-1]
 			line.Flags = lastLine.Flags & ^Revision & ^Elided
 		}
@@ -187,7 +180,7 @@ func (row *Row) Last(flag RowLineFlags) *GraphRowLine {
 	return &GraphRowLine{}
 }
 
-func isChangeIdLike(s string) bool {
+func isChangeIDLike(s string) bool {
 	for _, r := range s {
 		if !unicode.IsLetter(r) {
 			return false
@@ -196,15 +189,6 @@ func isChangeIdLike(s string) bool {
 	return true
 }
 
-func isHexLike(s string) bool {
-	for _, r := range s {
-		if !unicode.Is(unicode.Hex_Digit, r) {
-			return false
-		}
-	}
-	return true
-}
-
 func (row *Row) RowLinesIter(predicate RowLinesIteratorPredicate) func(yield func(index int, line *GraphRowLine) bool) {
 	return func(yield func(index int, line *GraphRowLine) bool) {
 		for i := range row.Lines {
