@@ -6,6 +6,8 @@ import (
 	"slices"
 	"strings"
 
+	"github.com/charmbracelet/lipgloss"
+	"github.com/charmbracelet/x/cellbuf"
 	"github.com/idursun/jjui/internal/ui/common/menu"
 
 	"github.com/charmbracelet/bubbles/key"
@@ -21,7 +23,10 @@ type updateItemsMsg struct {
 	items []list.Item
 }
 
+var _ common.Model = (*Model)(nil)
+
 type Model struct {
+	*common.ViewNode
 	context     *context.MainContext
 	current     *jj.Commit
 	menu        menu.Menu
@@ -29,20 +34,21 @@ type Model struct {
 	distanceMap map[string]int
 }
 
-func (m *Model) Width() int {
-	return m.menu.Width()
+func (m *Model) ShortHelp() []key.Binding {
+	return []key.Binding{
+		m.keymap.Cancel,
+		m.keymap.Apply,
+		m.keymap.Bookmark.Move,
+		m.keymap.Bookmark.Delete,
+		m.keymap.Bookmark.Forget,
+		m.keymap.Bookmark.Track,
+		m.keymap.Bookmark.Untrack,
+		m.menu.List.KeyMap.Filter,
+	}
 }
 
-func (m *Model) Height() int {
-	return m.menu.Height()
-}
-
-func (m *Model) SetWidth(w int) {
-	m.menu.SetWidth(w)
-}
-
-func (m *Model) SetHeight(h int) {
-	m.menu.SetHeight(h)
+func (m *Model) FullHelp() [][]key.Binding {
+	return [][]key.Binding{m.ShortHelp()}
 }
 
 type commandType int
@@ -85,8 +91,8 @@ func (m *Model) Init() tea.Cmd {
 	return tea.Batch(m.loadAll, m.loadMovables)
 }
 
-func (m *Model) filtered(filter string) (tea.Model, tea.Cmd) {
-	return m, m.menu.Filtered(filter)
+func (m *Model) filtered(filter string) tea.Cmd {
+	return m.menu.Filtered(filter)
 }
 
 func (m *Model) loadMovables() tea.Msg {
@@ -170,7 +176,7 @@ func (m *Model) loadAll() tea.Msg {
 	}
 }
 
-func (m *Model) Update(msg tea.Msg) (tea.Model, tea.Cmd) {
+func (m *Model) Update(msg tea.Msg) tea.Cmd {
 	switch msg := msg.(type) {
 	case tea.KeyMsg:
 		if m.menu.List.SettingFilter() {
@@ -182,13 +188,13 @@ func (m *Model) Update(msg tea.Msg) (tea.Model, tea.Cmd) {
 				m.menu.List.ResetFilter()
 				return m.filtered("")
 			}
-			return m, common.Close
+			return common.Close
 		case key.Matches(msg, m.keymap.Apply):
 			if m.menu.List.SelectedItem() == nil {
 				break
 			}
 			action := m.menu.List.SelectedItem().(item)
-			return m, m.context.RunCommand(action.args, common.Refresh, common.Close)
+			return m.context.RunCommand(action.args, common.Refresh, common.Close)
 		case key.Matches(msg, m.keymap.Bookmark.Move) && m.menu.Filter != "move":
 			return m.filtered("move")
 		case key.Matches(msg, m.keymap.Bookmark.Delete) && m.menu.Filter != "delete":
@@ -202,18 +208,18 @@ func (m *Model) Update(msg tea.Msg) (tea.Model, tea.Cmd) {
 		default:
 			for _, listItem := range m.menu.List.Items() {
 				if item, ok := listItem.(item); ok && m.menu.Filter != "" && item.key == msg.String() {
-					return m, m.context.RunCommand(jj.Args(item.args...), common.Refresh, common.Close)
+					return m.context.RunCommand(jj.Args(item.args...), common.Refresh, common.Close)
 				}
 			}
 		}
 	case updateItemsMsg:
 		m.menu.Items = append(m.menu.Items, msg.items...)
 		slices.SortFunc(m.menu.Items, itemSorter)
-		return m, m.menu.List.SetItems(m.menu.Items)
+		return m.menu.List.SetItems(m.menu.Items)
 	}
 	var cmd tea.Cmd
 	m.menu.List, cmd = m.menu.List.Update(msg)
-	return m, cmd
+	return cmd
 }
 
 func itemSorter(a list.Item, b list.Item) int {
@@ -235,15 +241,14 @@ func itemSorter(a list.Item, b list.Item) int {
 }
 
 func (m *Model) View() string {
-	helpKeys := []key.Binding{
-		m.keymap.Bookmark.Move,
-		m.keymap.Bookmark.Delete,
-		m.keymap.Bookmark.Forget,
-		m.keymap.Bookmark.Track,
-		m.keymap.Bookmark.Untrack,
-	}
-
-	return m.menu.View(helpKeys)
+	m.menu.SetFrame(cellbuf.Rect(0, 0, 80, 40))
+	v := m.menu.View()
+	w, h := lipgloss.Size(v)
+	pw, ph := m.Parent.Width, m.Parent.Height
+	sx := (pw - w) / 2
+	sy := (ph - h) / 2
+	m.SetFrame(cellbuf.Rect(sx, sy, w, h))
+	return v
 }
 
 func (m *Model) distance(commitId string) int {
@@ -253,25 +258,25 @@ func (m *Model) distance(commitId string) int {
 	return math.MinInt32
 }
 
-func NewModel(c *context.MainContext, current *jj.Commit, commitIds []string, width int, height int) *Model {
+func NewModel(c *context.MainContext, current *jj.Commit, commitIds []string) *Model {
 	var items []list.Item
 	keymap := config.Current.GetKeyMap()
 
-	menu := menu.NewMenu(items, width, height, keymap, menu.WithStylePrefix("bookmarks"))
+	menu := menu.NewMenu(items, keymap, menu.WithStylePrefix("bookmarks"))
 	menu.Title = "Bookmark Operations"
 	menu.FilterMatches = func(i list.Item, filter string) bool {
 		return strings.HasPrefix(i.FilterValue(), filter)
 	}
 
 	m := &Model{
+		ViewNode:    common.NewViewNode(0, 0),
 		context:     c,
 		keymap:      keymap,
 		menu:        menu,
 		current:     current,
 		distanceMap: calcDistanceMap(current.CommitId, commitIds),
 	}
-	m.SetWidth(width)
-	m.SetHeight(height)
+	menu.Parent = m.ViewNode
 	return m
 }
 
