@@ -37,15 +37,26 @@ type (
 		Line string
 		Mode ExecMode
 	}
+	ExecProcessCompletedMsg struct {
+		Err error
+		Msg ExecMsg
+	}
 	FileSearchMsg struct {
 		Revset       string
 		PreviewShown bool
 		Commit       *jj.Commit
 		RawFileOut   []byte // raw output from `jj file list`
 	}
-	ShowPreview     bool
-	JumpToParentMsg struct {
-		Commit *jj.Commit
+	ShowPreview             bool
+	StartSquashOperationMsg struct {
+		Revision *jj.Commit
+		Files    []string
+	}
+	// DeferredUpdateMsg is used to defer an update until the next render cycle
+	// However, this is very hacky and should only be used for single model updates that doesn't interact with any other model
+	// Rule of thumb is that if you are updating a model with a public message, then you probably shouldn't use this
+	DeferredUpdateMsg struct {
+		Fn func() tea.Cmd
 	}
 )
 
@@ -107,15 +118,6 @@ func FileSearch(revset string, preview bool, commit *jj.Commit, rawFileOut []byt
 	}
 }
 
-func JumpToParent(commit *jj.Commit) tea.Cmd {
-	return func() tea.Msg {
-		if commit == nil {
-			return nil
-		}
-		return JumpToParentMsg{Commit: commit}
-	}
-}
-
 type ExecMode struct {
 	Mode   string
 	Prompt string
