@@ -1,6 +1,7 @@
 package describe
 
 import (
+	"github.com/charmbracelet/bubbles/cursor"
 	"github.com/charmbracelet/bubbles/key"
 	"github.com/charmbracelet/bubbles/textarea"
 	tea "github.com/charmbracelet/bubbletea"
@@ -12,21 +13,24 @@ import (
 	"github.com/idursun/jjui/internal/ui/operations"
 )
 
-var _ operations.Operation = (*Operation)(nil)
-var _ common.Editable = (*Operation)(nil)
+var (
+	_ operations.Operation = (*Operation)(nil)
+	_ common.Editable      = (*Operation)(nil)
+)
 
 type Operation struct {
+	*common.ViewNode
 	context  *context.MainContext
 	keyMap   config.KeyMappings[key.Binding]
 	input    textarea.Model
 	revision string
 }
 
-func (o Operation) IsEditing() bool {
+func (o *Operation) IsEditing() bool {
 	return true
 }
 
-func (o Operation) ShortHelp() []key.Binding {
+func (o *Operation) ShortHelp() []key.Binding {
 	return []key.Binding{
 		o.keyMap.Cancel,
 		o.keyMap.InlineDescribe.Editor,
@@ -34,81 +38,75 @@ func (o Operation) ShortHelp() []key.Binding {
 	}
 }
 
-func (o Operation) FullHelp() [][]key.Binding {
+func (o *Operation) FullHelp() [][]key.Binding {
 	return [][]key.Binding{o.ShortHelp()}
 }
 
-func (o Operation) Width() int {
-	return o.input.Width()
-}
-
-func (o Operation) Height() int {
-	return o.input.Height()
-}
-
-func (o Operation) SetWidth(w int) {
-	o.input.SetWidth(w)
-}
-
-func (o Operation) SetHeight(h int) {
-	o.input.SetHeight(h)
-}
-
-func (o Operation) IsFocused() bool {
+func (o *Operation) IsFocused() bool {
 	return true
 }
 
-func (o Operation) Render(commit *jj.Commit, pos operations.RenderPosition) string {
+func (o *Operation) Render(commit *jj.Commit, pos operations.RenderPosition) string {
 	if pos != operations.RenderOverDescription {
 		return ""
 	}
 	return o.View()
 }
 
-func (o Operation) Name() string {
+func (o *Operation) Name() string {
 	return "desc"
 }
 
-func (o Operation) Update(msg tea.Msg) (tea.Model, tea.Cmd) {
+func (o *Operation) Update(msg tea.Msg) tea.Cmd {
+	// ignore cursor blink messages to prevent unnecessary rendering and height
+	// recalculations
+	var cmd tea.Cmd
+	if _, ok := msg.(cursor.BlinkMsg); ok {
+		o.input, cmd = o.input.Update(msg)
+		return cmd
+	}
+
 	if keyMsg, ok := msg.(tea.KeyMsg); ok {
 		switch {
 		case key.Matches(keyMsg, o.keyMap.Cancel):
-			return o, common.Close
+			return common.Close
 		case key.Matches(keyMsg, o.keyMap.InlineDescribe.Editor):
 			commit := &jj.Commit{
 				ChangeId: o.revision,
 			}
 			selectedRevisions := jj.NewSelectedRevisions(commit)
-			return o, o.context.RunCommand(
+			return o.context.RunCommand(
 				jj.SetDescription(o.revision, o.input.Value()),
 				common.Close,
 				o.context.RunInteractiveCommand(jj.Describe(selectedRevisions), common.Refresh),
 			)
 		case key.Matches(keyMsg, o.keyMap.InlineDescribe.Accept):
-			return o, o.context.RunCommand(jj.SetDescription(o.revision, o.input.Value()), common.Close, common.Refresh)
+			return o.context.RunCommand(jj.SetDescription(o.revision, o.input.Value()), common.Close, common.Refresh)
 		}
 	}
-	var cmd tea.Cmd
 	o.input, cmd = o.input.Update(msg)
 
 	newValue := o.input.Value()
 	h := lipgloss.Height(newValue)
 	if h >= o.input.Height() {
-		o.input.SetHeight(h + 1)
+		o.SetHeight(h + 1)
 	}
 
-	return o, cmd
+	return cmd
 }
 
-func (o Operation) Init() tea.Cmd {
+func (o *Operation) Init() tea.Cmd {
 	return nil
 }
 
-func (o Operation) View() string {
+func (o *Operation) View() string {
+	o.SetWidth(o.Parent.Width)
+	o.input.SetWidth(o.Width)
+	o.input.SetHeight(o.Height)
 	return o.input.View()
 }
 
-func NewOperation(context *context.MainContext, revision string, width int) Operation {
+func NewOperation(context *context.MainContext, revision string) *Operation {
 	descOutput, _ := context.RunCommandImmediate(jj.GetDescription(revision))
 	desc := string(descOutput)
 	h := lipgloss.Height(desc)
@@ -123,11 +121,10 @@ func NewOperation(context *context.MainContext, revision string, width int) Oper
 	input.FocusedStyle.Base = selectedStyle.Underline(false).Strikethrough(false).Reverse(false).Blink(false)
 	input.FocusedStyle.CursorLine = input.FocusedStyle.Base
 	input.SetValue(desc)
-	input.SetHeight(h + 1)
-	input.SetWidth(width)
 	input.Focus()
 
-	return Operation{
+	return &Operation{
+		ViewNode: common.NewViewNode(0, h+1),
 		context:  context,
 		keyMap:   config.Current.GetKeyMap(),
 		input:    input,
