@@ -38,7 +38,6 @@ type Operation struct {
 	keymap            config.KeyMappings[key.Binding]
 	targetMarkerStyle lipgloss.Style
 	revision          *jj.Commit
-	height            int
 	confirmation      *confirmation.Model
 	keyMap            config.KeyMappings[key.Binding]
 	styles            styles
@@ -56,15 +55,15 @@ func (s *Operation) Init() tea.Cmd {
 	return s.load(s.revision.GetChangeId())
 }
 
-func (s *Operation) Update(msg tea.Msg) (tea.Model, tea.Cmd) {
+func (s *Operation) Update(msg tea.Msg) tea.Cmd {
 	switch msg := msg.(type) {
 	case confirmation.CloseMsg:
 		s.confirmation = nil
 		s.selectedHint = ""
 		s.unselectedHint = ""
-		return s, nil
+		return nil
 	case common.RefreshMsg:
-		return s, s.load(s.revision.GetChangeId())
+		return s.load(s.revision.GetChangeId())
 	case updateCommitStatusMsg:
 		items := s.createListItems(msg.summary, msg.selectedFiles)
 		var selectionChangedCmd tea.Cmd
@@ -87,54 +86,53 @@ func (s *Operation) Update(msg tea.Msg) (tea.Model, tea.Cmd) {
 			selectionChangedCmd = s.context.SetSelectedItem(first)
 		}
 		s.setItems(items)
-		return s, selectionChangedCmd
+		return selectionChangedCmd
 	default:
 		oldCursor := s.cursor
-		var cmd tea.Cmd
-		var newModel *Operation
-		newModel, cmd = s.internalUpdate(msg)
+		var cmds []tea.Cmd
+		cmds = append(cmds, s.internalUpdate(msg))
 		if s.cursor != oldCursor {
-			cmd = tea.Batch(cmd, s.context.SetSelectedItem(context.SelectedFile{
+			cmds = append(cmds, s.context.SetSelectedItem(context.SelectedFile{
 				ChangeId: s.revision.GetChangeId(),
 				CommitId: s.revision.CommitId,
 				File:     s.current().fileName,
 			}))
 		}
-		return newModel, cmd
+		return tea.Batch(cmds...)
 	}
 }
 
-func (s *Operation) internalUpdate(msg tea.Msg) (*Operation, tea.Cmd) {
+func (s *Operation) internalUpdate(msg tea.Msg) tea.Cmd {
 	switch msg := msg.(type) {
 	case tea.KeyMsg:
 		if s.confirmation != nil {
-			model, cmd := s.confirmation.Update(msg)
-			s.confirmation = model
-			return s, cmd
+			return s.confirmation.Update(msg)
 		}
 		switch {
 		case key.Matches(msg, s.keyMap.Up):
 			s.cursorUp()
-			return s, nil
+			return nil
 		case key.Matches(msg, s.keyMap.Down):
 			s.cursorDown()
-			return s, nil
+			return nil
 		case key.Matches(msg, s.keyMap.Cancel), key.Matches(msg, s.keyMap.Details.Close):
-			return s, common.Close
+			return common.Close
+		case key.Matches(msg, s.keyMap.Quit): // handle global quit after cancel
+			return tea.Quit
 		case key.Matches(msg, s.keyMap.Refresh):
-			return s, common.Refresh
+			return common.Refresh
 		case key.Matches(msg, s.keyMap.Details.Diff):
 			selected := s.current()
 			if selected == nil {
-				return s, nil
+				return nil
 			}
-			return s, func() tea.Msg {
+			return func() tea.Msg {
 				output, _ := s.context.RunCommandImmediate(jj.Diff(s.revision.GetChangeId(), selected.fileName))
 				return common.ShowDiffMsg(output)
 			}
 		case key.Matches(msg, s.keyMap.Details.Split, s.keyMap.Details.SplitParallel):
 			isParallel := key.Matches(msg, s.keyMap.Details.SplitParallel)
-			selectedFiles := s.getSelectedFiles()
+			selectedFiles := s.getSelectedFiles(true)
 			s.selectedHint = "stays as is"
 			s.unselectedHint = "moves to the new revision"
 			model := confirmation.New(
@@ -148,13 +146,13 @@ func (s *Operation) internalUpdate(msg tea.Msg) (*Operation, tea.Cmd) {
 					key.NewBinding(key.WithKeys("n", "esc"), key.WithHelp("n/esc", "no"))),
 			)
 			s.confirmation = model
-			return s, s.confirmation.Init()
+			return s.confirmation.Init()
 		case key.Matches(msg, s.keyMap.Details.Squash):
-			return s, func() tea.Msg {
-				return common.StartSquashOperationMsg{Revision: s.revision, Files: s.getSelectedFiles()}
+			return func() tea.Msg {
+				return common.StartSquashOperationMsg{Revision: s.revision, Files: s.getSelectedFiles(true)}
 			}
 		case key.Matches(msg, s.keyMap.Details.Restore):
-			selectedFiles := s.getSelectedFiles()
+			selectedFiles := s.getSelectedFiles(true)
 			selected := s.current()
 			s.selectedHint = "gets restored"
 			s.unselectedHint = "stays as is"
@@ -172,9 +170,9 @@ func (s *Operation) internalUpdate(msg tea.Msg) (*Operation, tea.Cmd) {
 					key.NewBinding(key.WithKeys("n", "esc"), key.WithHelp("n/esc", "no"))),
 			)
 			s.confirmation = model
-			return s, s.confirmation.Init()
+			return s.confirmation.Init()
 		case key.Matches(msg, s.keyMap.Details.Absorb):
-			selectedFiles := s.getSelectedFiles()
+			selectedFiles := s.getSelectedFiles(true)
 			s.selectedHint = "might get absorbed into parents"
 			s.unselectedHint = "stays as is"
 			model := confirmation.New(
@@ -188,7 +186,7 @@ func (s *Operation) internalUpdate(msg tea.Msg) (*Operation, tea.Cmd) {
 					key.NewBinding(key.WithKeys("n", "esc"), key.WithHelp("n/esc", "no"))),
 			)
 			s.confirmation = model
-			return s, s.confirmation.Init()
+			return s.confirmation.Init()
 		case key.Matches(msg, s.keyMap.Details.ToggleSelect):
 			if current := s.current(); current != nil {
 				isChecked := !current.selected
@@ -207,14 +205,14 @@ func (s *Operation) internalUpdate(msg tea.Msg) (*Operation, tea.Cmd) {
 
 				s.cursorDown()
 			}
-			return s, nil
+			return nil
 		case key.Matches(msg, s.keyMap.Details.RevisionsChangingFile):
 			if current := s.current(); current != nil {
-				return s, tea.Batch(common.Close, common.UpdateRevSet(fmt.Sprintf("files(%s)", jj.EscapeFileName(current.fileName))))
+				return tea.Batch(common.Close, common.UpdateRevSet(fmt.Sprintf("files(%s)", jj.EscapeFileName(current.fileName))))
 			}
 		}
 	}
-	return s, nil
+	return nil
 }
 
 func (s *Operation) View() string {
@@ -227,7 +225,7 @@ func (s *Operation) View() string {
 	if s.Len() == 0 {
 		return s.styles.Dimmed.Render("No changes\n")
 	}
-	s.SetHeight(min(s.height-5-ch, s.Len()))
+	s.SetHeight(min(s.Parent.Height-5-ch, s.Len()))
 	filesView := s.renderer.Render(s.cursor)
 
 	view := lipgloss.JoinVertical(lipgloss.Top, filesView, confirmationView)
@@ -245,8 +243,9 @@ func (s *Operation) View() string {
 	return lipgloss.Place(w, h, 0, 0, view, lipgloss.WithWhitespaceBackground(s.styles.Text.GetBackground()))
 }
 
-func (s *Operation) SetSelectedRevision(commit *jj.Commit) {
+func (s *Operation) SetSelectedRevision(commit *jj.Commit) tea.Cmd {
 	s.Current = commit
+	return nil
 }
 
 func (s *Operation) ShortHelp() []key.Binding {
@@ -282,7 +281,7 @@ func (s *Operation) Name() string {
 	return "details"
 }
 
-func (s *Operation) getSelectedFiles() []string {
+func (s *Operation) getSelectedFiles(allowVirtualSelection bool) []string {
 	selectedFiles := make([]string, 0)
 	if len(s.files) == 0 {
 		return selectedFiles
@@ -293,7 +292,7 @@ func (s *Operation) getSelectedFiles() []string {
 			selectedFiles = append(selectedFiles, f.fileName)
 		}
 	}
-	if len(selectedFiles) == 0 {
+	if len(selectedFiles) == 0 && allowVirtualSelection {
 		selectedFiles = append(selectedFiles, s.current().fileName)
 		return selectedFiles
 	}
@@ -303,16 +302,18 @@ func (s *Operation) getSelectedFiles() []string {
 func (s *Operation) createListItems(content string, selectedFiles []string) []*item {
 	var items []*item
 	scanner := bufio.NewScanner(strings.NewReader(content))
+	scanner.Split(bufio.ScanWords)
 	var conflicts []bool
-	if scanner.Scan() {
-		conflictsLine := strings.Split(scanner.Text(), " ")
-		for _, c := range conflictsLine {
-			conflicts = append(conflicts, c == "true")
+	for scanner.Scan() {
+		field := scanner.Text()
+		if field == "$" {
+			break
 		}
-	} else {
-		return items
+		conflicts = append(conflicts, field == "true")
 	}
 
+	start := strings.IndexByte(content, '$')
+	scanner = bufio.NewScanner(strings.NewReader(content[start+1:]))
 	index := 0
 	for scanner.Scan() {
 		file := strings.TrimSpace(scanner.Text())
@@ -358,7 +359,7 @@ func (s *Operation) load(revision string) tea.Cmd {
 		if err == nil {
 			return func() tea.Msg {
 				summary := string(output)
-				selectedFiles := s.getSelectedFiles()
+				selectedFiles := s.getSelectedFiles(false)
 				return updateCommitStatusMsg{summary, selectedFiles}
 			}
 		}
@@ -371,7 +372,7 @@ func (s *Operation) load(revision string) tea.Cmd {
 	}
 }
 
-func NewOperation(context *context.MainContext, selected *jj.Commit, height int) *Operation {
+func NewOperation(context *context.MainContext, selected *jj.Commit) *Operation {
 	keyMap := config.Current.GetKeyMap()
 
 	s := styles{
@@ -386,16 +387,16 @@ func NewOperation(context *context.MainContext, selected *jj.Commit, height int)
 		Conflict: common.DefaultPalette.Get("revisions details conflict"),
 	}
 
-	l := NewDetailsList(s, common.NewSizeable(0, height))
+	l := NewDetailsList(s, common.NewViewNode(0, 0))
 	op := &Operation{
 		DetailsList:       l,
 		context:           context,
 		revision:          selected,
 		keyMap:            keyMap,
 		styles:            s,
-		height:            height,
 		keymap:            config.Current.GetKeyMap(),
 		targetMarkerStyle: common.DefaultPalette.Get("revisions details target_marker"),
 	}
+	l.Parent = op.ViewNode
 	return op
 }
