@@ -58,11 +58,11 @@ func (r *Operation) Init() tea.Cmd {
 	return nil
 }
 
-func (r *Operation) Update(msg tea.Msg) (tea.Model, tea.Cmd) {
+func (r *Operation) Update(msg tea.Msg) tea.Cmd {
 	if msg, ok := msg.(tea.KeyMsg); ok {
-		return r, r.HandleKey(msg)
+		return r.HandleKey(msg)
 	}
-	return r, nil
+	return nil
 }
 
 func (r *Operation) View() string {
@@ -86,8 +86,9 @@ func (r *Operation) HandleKey(msg tea.KeyMsg) tea.Cmd {
 	return nil
 }
 
-func (r *Operation) SetSelectedRevision(commit *jj.Commit) {
+func (r *Operation) SetSelectedRevision(commit *jj.Commit) tea.Cmd {
 	r.To = commit
+	return nil
 }
 
 func (r *Operation) ShortHelp() []key.Binding {
