@@ -36,23 +36,29 @@ import (
 	"github.com/idursun/jjui/internal/ui/operations/squash"
 )
 
-var _ list.IList = (*Model)(nil)
-var _ list.IListCursor = (*Model)(nil)
-var _ common.Focusable = (*Model)(nil)
-var _ common.Editable = (*Model)(nil)
+var (
+	_ list.IList         = (*Model)(nil)
+	_ list.IListCursor   = (*Model)(nil)
+	_ common.Focusable   = (*Model)(nil)
+	_ common.Editable    = (*Model)(nil)
+	_ common.IMouseAware = (*Model)(nil)
+)
 
-var pageDownKey = key.NewBinding(key.WithKeys("pgdown"))
-var pageUpKey = key.NewBinding(key.WithKeys("pgup"))
+var (
+	pageDownKey = key.NewBinding(key.WithKeys("pgdown"))
+	pageUpKey   = key.NewBinding(key.WithKeys("pgup"))
+)
 
 type Model struct {
-	*common.Sizeable
+	*common.ViewNode
+	*common.MouseAware
 	rows             []parser.Row
 	tag              atomic.Uint64
 	revisionToSelect string
 	offScreenRows    []parser.Row
 	streamer         *graph.GraphStreamer
 	hasMore          bool
-	op               tea.Model
+	op               common.Model
 	cursor           int
 	context          *appContext.MainContext
 	keymap           config.KeyMappings[key.Binding]
@@ -65,63 +71,8 @@ type Model struct {
 	textStyle        lipgloss.Style
 	dimmedStyle      lipgloss.Style
 	selectedStyle    lipgloss.Style
-}
-
-func (m *Model) Cursor() int {
-	return m.cursor
-}
-
-func (m *Model) SetCursor(index int) {
-	if index >= 0 && index < len(m.rows) {
-		m.cursor = index
-	}
-}
-
-func (m *Model) Len() int {
-	return len(m.rows)
-}
-
-func (m *Model) GetItemRenderer(index int) list.IItemRenderer {
-	var (
-		before, after, renderOverDescription, beforeCommitId, beforeChangeId string
-	)
-	row := m.rows[index]
-	inLane := m.renderer.tracer.IsInSameLane(index)
-	isHighlighted := index == m.cursor
-
-	if op, ok := m.op.(operations.Operation); ok {
-		before = op.Render(row.Commit, operations.RenderPositionBefore)
-		after = op.Render(row.Commit, operations.RenderPositionAfter)
-		renderOverDescription = ""
-		if isHighlighted {
-			renderOverDescription = op.Render(row.Commit, operations.RenderOverDescription)
-		}
-		beforeCommitId = op.Render(row.Commit, operations.RenderBeforeCommitId)
-		beforeChangeId = op.Render(row.Commit, operations.RenderBeforeChangeId)
-	}
-
-	return &itemRenderer{
-		row:            row,
-		before:         before,
-		after:          after,
-		description:    renderOverDescription,
-		beforeChangeId: beforeChangeId,
-		beforeCommitId: beforeCommitId,
-		isHighlighted:  isHighlighted,
-		SearchText:     m.quickSearch,
-		textStyle:      m.textStyle,
-		dimmedStyle:    m.dimmedStyle,
-		selectedStyle:  m.selectedStyle,
-		isChecked:      m.renderer.selections[row.Commit.GetChangeId()],
-		isGutterInLane: func(lineIndex, segmentIndex int) bool {
-			return m.renderer.tracer.IsGutterInLane(index, lineIndex, segmentIndex)
-		},
-		updateGutterText: func(lineIndex, segmentIndex int, text string) string {
-			return m.renderer.tracer.UpdateGutterText(index, lineIndex, segmentIndex, text)
-		},
-		inLane: inLane,
-		op:     m.op.(operations.Operation),
-	}
+	ensureCursorView bool
+	requestInFlight  bool
 }
 
 type revisionsMsg struct {
@@ -151,6 +102,102 @@ type appendRowsBatchMsg struct {
 	tag     uint64
 }
 
+func (m *Model) Cursor() int {
+	return m.cursor
+}
+
+func (m *Model) SetCursor(index int) {
+	if index >= 0 && index < len(m.rows) {
+		m.cursor = index
+		m.ensureCursorView = true
+	}
+}
+
+func (m *Model) ClickAt(x, y int) tea.Cmd {
+	if len(m.rows) == 0 {
+		return nil
+	}
+
+	localY := y - m.Frame.Min.Y
+
+	currentStart := m.renderer.ViewRange.Start
+	if localY >= m.Height {
+		localY = m.Height - 1
+		if localY < 0 {
+			return nil
+		}
+	}
+	line := currentStart + localY
+	row := m.rowAtLine(line)
+	if row == -1 {
+		return nil
+	}
+
+	m.SetCursor(row)
+	return m.updateSelection()
+}
+
+func (m *Model) rowAtLine(line int) int {
+	for _, rr := range m.renderer.RowRanges() {
+		if line >= rr.StartLine && line < rr.EndLine {
+			return rr.Row
+		}
+	}
+	return -1
+}
+
+func (m *Model) Scroll(delta int) tea.Cmd {
+	m.ensureCursorView = false
+	desiredStart := m.renderer.ViewRange.Start + delta
+	if desiredStart < 0 {
+		desiredStart = 0
+	}
+
+	totalLines := m.renderer.AbsoluteLineCount()
+	maxStart := totalLines - m.Height
+	if maxStart < 0 {
+		maxStart = 0
+	}
+	newStart := desiredStart
+	if newStart > maxStart {
+		newStart = maxStart
+	}
+	m.renderer.ViewRange.Start = newStart
+
+	if m.hasMore && (desiredStart > maxStart || newStart+m.Height >= totalLines-1) {
+		return m.requestMoreRows(m.tag.Load())
+	}
+	return nil
+}
+
+func (m *Model) Len() int {
+	return len(m.rows)
+}
+
+func (m *Model) GetItemRenderer(index int) list.IItemRenderer {
+	row := m.rows[index]
+	inLane := m.renderer.tracer.IsInSameLane(index)
+	isHighlighted := index == m.cursor
+
+	return &itemRenderer{
+		row:           row,
+		isHighlighted: isHighlighted,
+		SearchText:    m.quickSearch,
+		textStyle:     m.textStyle,
+		dimmedStyle:   m.dimmedStyle,
+		selectedStyle: m.selectedStyle,
+		isChecked:     m.renderer.selections[row.Commit.GetChangeId()],
+		isGutterInLane: func(lineIndex, segmentIndex int) bool {
+			return m.renderer.tracer.IsGutterInLane(index, lineIndex, segmentIndex)
+		},
+		updateGutterText: func(lineIndex, segmentIndex int, text string) string {
+			return m.renderer.tracer.UpdateGutterText(index, lineIndex, segmentIndex, text)
+		},
+		inLane: inLane,
+		op:     m.op.(operations.Operation),
+	}
+}
+
 func (m *Model) IsEditing() bool {
 	if f, ok := m.op.(common.Editable); ok {
 		return f.IsEditing()
@@ -217,63 +264,79 @@ func (m *Model) Init() tea.Cmd {
 	return common.RefreshAndSelect("@")
 }
 
-func (m *Model) Update(msg tea.Msg) (*Model, tea.Cmd) {
+func (m *Model) Update(msg tea.Msg) tea.Cmd {
 	if k, ok := msg.(revisionsMsg); ok {
 		msg = k.msg
 	}
-	var cmd tea.Cmd
-	var nm *Model
-	nm, cmd = m.internalUpdate(msg)
+	cmd := m.internalUpdate(msg)
 
 	if curSelected := m.SelectedRevision(); curSelected != nil {
 		if op, ok := m.op.(operations.TracksSelectedRevision); ok {
-			op.SetSelectedRevision(curSelected)
+			cmd = tea.Batch(cmd, op.SetSelectedRevision(curSelected))
 		}
 	}
 
-	return nm, cmd
+	return cmd
 }
 
-func (m *Model) internalUpdate(msg tea.Msg) (*Model, tea.Cmd) {
+func (m *Model) internalUpdate(msg tea.Msg) tea.Cmd {
 	switch msg := msg.(type) {
+	//case rebase.updateHighlightedIdsMsg:
+	//	return m.op.Update(msg)
+	case tea.MouseMsg:
+		switch msg.Action {
+		case tea.MouseActionPress:
+			switch msg.Button {
+			case tea.MouseButtonLeft:
+				if !m.InNormalMode() {
+					return nil
+				}
+				return m.ClickAt(msg.X, msg.Y)
+			case tea.MouseButtonWheelUp:
+				return m.Scroll(-3)
+			case tea.MouseButtonWheelDown:
+				return m.Scroll(3)
+			}
+			return nil
+		}
+
 	case common.CloseViewMsg:
 		m.op = operations.NewDefault()
-		return m, m.updateSelection()
+		return m.updateSelection()
 	case common.QuickSearchMsg:
 		m.quickSearch = string(msg)
-		m.cursor = m.search(0)
+		m.SetCursor(m.search(0))
 		m.op = operations.NewDefault()
 		m.renderer.Reset()
-		return m, nil
+		return nil
 	case common.CommandCompletedMsg:
 		m.output = msg.Output
 		m.err = msg.Err
-		return m, nil
+		return nil
 	case common.AutoRefreshMsg:
 		id, _ := m.context.RunCommandImmediate(jj.OpLogId(true))
 		currentOperationId := string(id)
 		log.Println("Previous operation ID:", m.previousOpLogId, "Current operation ID:", currentOperationId)
 		if currentOperationId != m.previousOpLogId {
 			m.previousOpLogId = currentOperationId
-			return m, common.RefreshAndKeepSelections
+			return common.RefreshAndKeepSelections
 		}
 	case common.RefreshMsg:
 		if !msg.KeepSelections {
 			m.context.ClearCheckedItems(reflect.TypeFor[appContext.SelectedRevision]())
 		}
 		m.isLoading = true
-		var cmd tea.Cmd
-		m.op, cmd = m.op.Update(msg)
+		cmd := m.op.Update(msg)
 		if config.Current.Revisions.LogBatching {
 			currentTag := m.tag.Add(1)
-			return m, tea.Batch(m.loadStreaming(m.context.CurrentRevset, msg.SelectedRevision, currentTag), cmd)
+			return tea.Batch(m.loadStreaming(m.context.CurrentRevset, msg.SelectedRevision, currentTag), cmd)
 		} else {
-			return m, tea.Batch(m.load(m.context.CurrentRevset, msg.SelectedRevision), cmd)
+			return tea.Batch(m.load(m.context.CurrentRevset, msg.SelectedRevision), cmd)
 		}
 	case updateRevisionsMsg:
 		m.isLoading = false
 		m.updateGraphRows(msg.rows, msg.selectedRevision)
-		return m, tea.Batch(m.highlightChanges, m.updateSelection(), func() tea.Msg {
+		return tea.Batch(m.highlightChanges, m.updateSelection(), func() tea.Msg {
 			return common.UpdateRevisionsSuccessMsg{}
 		})
 	case startRowsStreamingMsg:
@@ -290,10 +353,12 @@ func (m *Model) internalUpdate(msg tea.Msg) (*Model, tea.Cmd) {
 			}
 		}
 		log.Println("Starting streaming revisions message received with tag:", msg.tag, "revision to select:", msg.selectedRevision)
-		return m, m.requestMoreRows(msg.tag)
+		return m.requestMoreRows(msg.tag)
 	case appendRowsBatchMsg:
+		m.requestInFlight = false
+
 		if msg.tag != m.tag.Load() {
-			return m, nil
+			return nil
 		}
 		m.offScreenRows = append(m.offScreenRows, msg.rows...)
 		m.hasMore = msg.hasMore
@@ -302,7 +367,7 @@ func (m *Model) internalUpdate(msg tea.Msg) (*Model, tea.Cmd) {
 		if m.hasMore {
 			// keep requesting rows until we reach the initial load count or the current cursor position
 			if len(m.offScreenRows) < m.cursor+1 || len(m.offScreenRows) < m.renderer.ViewRange.LastRowIndex+1 {
-				return m, m.requestMoreRows(msg.tag)
+				return m.requestMoreRows(msg.tag)
 			}
 		} else if m.streamer != nil {
 			m.streamer.Close()
@@ -311,16 +376,16 @@ func (m *Model) internalUpdate(msg tea.Msg) (*Model, tea.Cmd) {
 		currentSelectedRevision := m.SelectedRevision()
 		m.rows = m.offScreenRows
 		if m.revisionToSelect != "" {
-			m.cursor = m.selectRevision(m.revisionToSelect)
+			m.SetCursor(m.selectRevision(m.revisionToSelect))
 			m.revisionToSelect = ""
 		}
 
 		if m.cursor == -1 && currentSelectedRevision != nil {
-			m.cursor = m.selectRevision(currentSelectedRevision.GetChangeId())
+			m.SetCursor(m.selectRevision(currentSelectedRevision.GetChangeId()))
 		}
 
 		if (m.cursor < 0 || m.cursor >= len(m.rows)) && len(m.rows) > 0 {
-			m.cursor = 0
+			m.SetCursor(0)
 		}
 
 		cmds := []tea.Cmd{m.highlightChanges, m.updateSelection()}
@@ -329,25 +394,21 @@ func (m *Model) internalUpdate(msg tea.Msg) (*Model, tea.Cmd) {
 				return common.UpdateRevisionsSuccessMsg{}
 			})
 		}
-		return m, tea.Batch(cmds...)
+		return tea.Batch(cmds...)
 	case common.StartSquashOperationMsg:
 		return m.startSquash(jj.NewSelectedRevisions(msg.Revision), msg.Files)
 	}
 
 	if len(m.rows) == 0 {
-		return m, nil
+		return nil
 	}
 
 	if op, ok := m.op.(common.Editable); ok && op.IsEditing() {
-		var cmd tea.Cmd
-		m.op, cmd = m.op.Update(msg)
-		return m, cmd
+		return m.op.Update(msg)
 	}
 
 	if op, ok := m.op.(common.Overlay); ok && op.IsOverlay() {
-		var cmd tea.Cmd
-		m.op, cmd = m.op.Update(msg)
-		return m, cmd
+		return m.op.Update(msg)
 	}
 
 	var cmd tea.Cmd
@@ -359,47 +420,63 @@ func (m *Model) internalUpdate(msg tea.Msg) (*Model, tea.Cmd) {
 			if key.Matches(msg, pageUpKey) && m.renderer.LastRowIndex > m.renderer.FirstRowIndex {
 				amount = m.renderer.LastRowIndex - m.renderer.FirstRowIndex - 1
 			}
-			if m.cursor-amount >= 0 {
-				m.cursor = m.cursor - amount
+			if m.cursor == 0 && key.Matches(msg, pageUpKey) {
+				return func() tea.Msg {
+					return common.CommandCompletedMsg{
+						Output: fmt.Sprintf("Already at the top of revset `%s`", m.context.CurrentRevset),
+						Err:    nil,
+					}
+				}
 			}
-			return m, m.updateSelection()
+			m.SetCursor(max(m.cursor-amount, 0))
+			return m.updateSelection()
 		case key.Matches(msg, m.keymap.Down, pageDownKey):
 			amount := 1
 			if key.Matches(msg, pageDownKey) && m.renderer.LastRowIndex > m.renderer.FirstRowIndex {
 				amount = m.renderer.LastRowIndex - m.renderer.FirstRowIndex - 1
 			}
-			if m.cursor < len(m.rows)-amount {
-				m.cursor = m.cursor + amount
-			} else if m.hasMore {
-				return m, m.requestMoreRows(m.tag.Load())
+			if len(m.rows) > 0 && m.cursor == len(m.rows)-1 && !m.hasMore && key.Matches(msg, pageDownKey) {
+				return func() tea.Msg {
+					return common.CommandCompletedMsg{
+						Output: fmt.Sprintf("Already at the bottom of revset `%s`", m.context.CurrentRevset),
+						Err:    nil,
+					}
+				}
 			}
-			return m, m.updateSelection()
+			if m.cursor < len(m.rows)-amount {
+				m.SetCursor(m.cursor + amount)
+			} else if m.hasMore {
+				return m.requestMoreRows(m.tag.Load())
+			} else if len(m.rows) > 0 {
+				m.SetCursor(len(m.rows) - 1)
+			}
+			m.ensureCursorView = true
+			return m.updateSelection()
 		case key.Matches(msg, m.keymap.JumpToParent):
 			m.jumpToParent(m.SelectedRevisions())
-			return m, m.updateSelection()
+			return m.updateSelection()
 		case key.Matches(msg, m.keymap.JumpToChildren):
 			immediate, _ := m.context.RunCommandImmediate(jj.GetFirstChild(m.SelectedRevision()))
 			index := m.selectRevision(string(immediate))
 			if index != -1 {
-				m.cursor = index
+				m.SetCursor(index)
 			}
-			return m, m.updateSelection()
+			return m.updateSelection()
 		case key.Matches(msg, m.keymap.JumpToWorkingCopy):
 			workingCopyIndex := m.selectRevision("@")
 			if workingCopyIndex != -1 {
-				m.cursor = workingCopyIndex
+				m.SetCursor(workingCopyIndex)
 			}
-			return m, m.updateSelection()
+			return m.updateSelection()
 		case key.Matches(msg, m.keymap.AceJump):
 			op := ace_jump.NewOperation(m, func(index int) parser.Row {
 				return m.rows[index]
 			}, m.renderer.FirstRowIndex, m.renderer.LastRowIndex)
 			m.op = op
-			return m, op.Init()
+			return op.Init()
 		default:
 			if op, ok := m.op.(common.Focusable); ok && op.IsFocused() {
-				m.op, cmd = m.op.Update(msg)
-				return m, cmd
+				return m.op.Update(msg)
 			}
 
 			switch {
@@ -411,89 +488,95 @@ func (m *Model) internalUpdate(msg tea.Msg) (*Model, tea.Cmd) {
 				immediate, _ := m.context.RunCommandImmediate(jj.GetParent(jj.NewSelectedRevisions(commit)))
 				parentIndex := m.selectRevision(string(immediate))
 				if parentIndex != -1 {
-					m.cursor = parentIndex
+					m.SetCursor(parentIndex)
 				}
 			case key.Matches(msg, m.keymap.Cancel):
 				m.op = operations.NewDefault()
 			case key.Matches(msg, m.keymap.QuickSearchCycle):
-				m.cursor = m.search(m.cursor + 1)
+				m.SetCursor(m.search(m.cursor + 1))
 				m.renderer.Reset()
-				return m, nil
+				return nil
 			case key.Matches(msg, m.keymap.Details.Mode):
-				m.op = details.NewOperation(m.context, m.SelectedRevision(), m.Height)
-				return m, m.op.Init()
+				model := details.NewOperation(m.context, m.SelectedRevision())
+				model.Parent = m.ViewNode
+				m.op = model
+				return m.op.Init()
 			case key.Matches(msg, m.keymap.InlineDescribe.Mode):
-				m.op = describe.NewOperation(m.context, m.SelectedRevision().GetChangeId(), m.Width)
-				return m, m.op.Init()
+				model := describe.NewOperation(m.context, m.SelectedRevision().GetChangeId())
+				model.Parent = m.ViewNode
+				m.op = model
+				return m.op.Init()
 			case key.Matches(msg, m.keymap.New):
-				return m, m.context.RunCommand(jj.New(m.SelectedRevisions()), common.RefreshAndSelect("@"))
+				return m.context.RunCommand(jj.New(m.SelectedRevisions()), common.RefreshAndSelect("@"))
 			case key.Matches(msg, m.keymap.Commit):
-				return m, m.context.RunInteractiveCommand(jj.CommitWorkingCopy(), common.Refresh)
+				return m.context.RunInteractiveCommand(jj.CommitWorkingCopy(), common.Refresh)
 			case key.Matches(msg, m.keymap.Edit, m.keymap.ForceEdit):
 				ignoreImmutable := key.Matches(msg, m.keymap.ForceEdit)
-				return m, m.context.RunCommand(jj.Edit(m.SelectedRevision().GetChangeId(), ignoreImmutable), common.Refresh)
+				return m.context.RunCommand(jj.Edit(m.SelectedRevision().GetChangeId(), ignoreImmutable), common.Refresh)
 			case key.Matches(msg, m.keymap.Diffedit):
 				changeId := m.SelectedRevision().GetChangeId()
-				return m, m.context.RunInteractiveCommand(jj.DiffEdit(changeId), common.Refresh)
+				return m.context.RunInteractiveCommand(jj.DiffEdit(changeId), common.Refresh)
 			case key.Matches(msg, m.keymap.Absorb):
 				changeId := m.SelectedRevision().GetChangeId()
-				return m, m.context.RunCommand(jj.Absorb(changeId), common.Refresh)
+				return m.context.RunCommand(jj.Absorb(changeId), common.Refresh)
 			case key.Matches(msg, m.keymap.Abandon):
 				selections := m.SelectedRevisions()
 				m.op = abandon.NewOperation(m.context, selections)
-				return m, m.op.Init()
+				return m.op.Init()
 			case key.Matches(msg, m.keymap.Bookmark.Set):
 				m.op = bookmark.NewSetBookmarkOperation(m.context, m.SelectedRevision().GetChangeId())
-				return m, m.op.Init()
+				return m.op.Init()
 			case key.Matches(msg, m.keymap.Split, m.keymap.SplitParallel):
 				isParallel := key.Matches(msg, m.keymap.SplitParallel)
 				currentRevision := m.SelectedRevision().GetChangeId()
-				return m, m.context.RunInteractiveCommand(jj.Split(currentRevision, []string{}, isParallel), common.Refresh)
+				return m.context.RunInteractiveCommand(jj.Split(currentRevision, []string{}, isParallel), common.Refresh)
 			case key.Matches(msg, m.keymap.Describe):
 				selections := m.SelectedRevisions()
-				return m, m.context.RunInteractiveCommand(jj.Describe(selections), common.Refresh)
+				return m.context.RunInteractiveCommand(jj.Describe(selections), common.Refresh)
 			case key.Matches(msg, m.keymap.Evolog.Mode):
-				m.op = evolog.NewOperation(m.context, m.SelectedRevision(), m.Width, m.Height)
-				return m, m.op.Init()
+				model := evolog.NewOperation(m.context, m.SelectedRevision())
+				model.Parent = m.ViewNode
+				m.op = model
+				return m.op.Init()
 			case key.Matches(msg, m.keymap.Diff):
-				return m, func() tea.Msg {
+				return func() tea.Msg {
 					changeId := m.SelectedRevision().GetChangeId()
 					output, _ := m.context.RunCommandImmediate(jj.Diff(changeId, ""))
 					return common.ShowDiffMsg(output)
 				}
 			case key.Matches(msg, m.keymap.Refresh):
-				return m, common.Refresh
+				return common.Refresh
 			case key.Matches(msg, m.keymap.Squash.Mode):
 				return m.startSquash(m.SelectedRevisions(), nil)
 			case key.Matches(msg, m.keymap.Revert.Mode):
 				m.op = revert.NewOperation(m.context, m.SelectedRevisions(), revert.TargetDestination)
-				return m, m.op.Init()
+				return m.op.Init()
 			case key.Matches(msg, m.keymap.Rebase.Mode):
 				m.op = rebase.NewOperation(m.context, m.SelectedRevisions(), rebase.SourceRevision, rebase.TargetDestination)
-				return m, m.op.Init()
+				return m.op.Init()
 			case key.Matches(msg, m.keymap.Duplicate.Mode):
 				m.op = duplicate.NewOperation(m.context, m.SelectedRevisions(), duplicate.TargetDestination)
-				return m, m.op.Init()
+				return m.op.Init()
 			case key.Matches(msg, m.keymap.SetParents):
 				m.op = set_parents.NewModel(m.context, m.SelectedRevision())
-				return m, m.op.Init()
+				return m.op.Init()
 			}
 		}
 	}
 
-	return m, cmd
+	return cmd
 }
 
-func (m *Model) startSquash(selectedRevisions jj.SelectedRevisions, files []string) (*Model, tea.Cmd) {
+func (m *Model) startSquash(selectedRevisions jj.SelectedRevisions, files []string) tea.Cmd {
 	parent, _ := m.context.RunCommandImmediate(jj.GetParent(selectedRevisions))
 	parentIdx := m.selectRevision(string(parent))
 	if parentIdx != -1 {
-		m.cursor = parentIdx
+		m.SetCursor(parentIdx)
 	} else if m.cursor < len(m.rows)-1 {
-		m.cursor++
+		m.SetCursor(m.cursor + 1)
 	}
 	m.op = squash.NewOperation(m.context, selectedRevisions, squash.WithFiles(files))
-	return m, m.op.Init()
+	return m.op.Init()
 }
 
 func (m *Model) updateSelection() tea.Cmd {
@@ -546,15 +629,15 @@ func (m *Model) updateGraphRows(rows []parser.Row, selectedRevision string) {
 	m.rows = rows
 
 	if len(m.rows) > 0 {
-		m.cursor = m.selectRevision(currentSelectedRevision)
+		m.SetCursor(m.selectRevision(currentSelectedRevision))
 		if m.cursor == -1 {
-			m.cursor = m.selectRevision("@")
+			m.SetCursor(m.selectRevision("@"))
 		}
 		if m.cursor == -1 {
-			m.cursor = 0
+			m.SetCursor(0)
 		}
 	} else {
-		m.cursor = 0
+		m.SetCursor(0)
 	}
 }
 
@@ -576,14 +659,13 @@ func (m *Model) View() string {
 
 	m.renderer.selections = m.context.GetSelectedRevisions()
 
-	output := m.renderer.Render(m.cursor)
-	output = m.textStyle.MaxWidth(m.Width).Render(output)
-	return lipgloss.Place(m.Width, m.Height, 0, 0, output)
+	output := m.renderer.RenderWithOptions(list.RenderOptions{FocusIndex: m.cursor, EnsureFocusVisible: m.ensureCursorView})
+	return output
 }
 
 func (m *Model) load(revset string, selectedRevision string) tea.Cmd {
 	return func() tea.Msg {
-		output, err := m.context.RunCommandImmediate(jj.Log(revset, config.Current.Limit))
+		output, err := m.context.RunCommandImmediate(jj.Log(revset, config.Current.Limit, m.context.JJConfig.Templates.Log))
 		if err != nil {
 			return common.UpdateRevisionsFailedMsg{
 				Err:    err,
@@ -600,45 +682,35 @@ func (m *Model) loadStreaming(revset string, selectedRevision string, tag uint64
 		return nil
 	}
 
-	if m.streamer != nil {
-		m.streamer.Close()
-		m.streamer = nil
-	}
-
-	m.hasMore = false
-
-	var notifyErrorCmd tea.Cmd
-	streamer, err := graph.NewGraphStreamer(m.context, revset)
+	streamer, err := graph.NewGraphStreamer(m.context, revset, m.context.JJConfig.Templates.Log)
 	if err != nil {
-		notifyErrorCmd = func() tea.Msg {
+		return func() tea.Msg {
 			return common.UpdateRevisionsFailedMsg{
 				Err:    err,
 				Output: fmt.Sprintf("%v", err),
 			}
 		}
 	}
+
+	if m.streamer != nil {
+		m.streamer.Close()
+		m.streamer = nil
+	}
 	m.streamer = streamer
 	m.hasMore = true
-	m.offScreenRows = nil
-	log.Println("Starting streaming revisions with tag:", tag)
-	startStreamingCmd := func() tea.Msg {
-		return startRowsStreamingMsg{selectedRevision, tag}
-	}
 
-	return tea.Batch(startStreamingCmd, notifyErrorCmd)
+	return m.Update(startRowsStreamingMsg{selectedRevision: selectedRevision, tag: tag})
 }
 
 func (m *Model) requestMoreRows(tag uint64) tea.Cmd {
-	return func() tea.Msg {
-		if m.streamer == nil || !m.hasMore {
-			return nil
-		}
-		if tag == m.tag.Load() {
-			batch := m.streamer.RequestMore()
-			return appendRowsBatchMsg{batch.Rows, batch.HasMore, tag}
-		}
+	if m.requestInFlight || m.streamer == nil || !m.hasMore || tag != m.tag.Load() {
 		return nil
 	}
+
+	m.requestInFlight = true
+	batch := m.streamer.RequestMore()
+	m.Update(appendRowsBatchMsg{batch.Rows, batch.HasMore, tag})
+	return nil
 }
 
 func (m *Model) selectRevision(revision string) int {
@@ -690,7 +762,8 @@ func (m *Model) GetCommitIds() []string {
 func New(c *appContext.MainContext) *Model {
 	keymap := config.Current.GetKeyMap()
 	m := Model{
-		Sizeable:      &common.Sizeable{Width: 0, Height: 0},
+		ViewNode:      common.NewViewNode(0, 0),
+		MouseAware:    common.NewMouseAware(),
 		context:       c,
 		keymap:        keymap,
 		rows:          nil,
@@ -701,7 +774,7 @@ func New(c *appContext.MainContext) *Model {
 		dimmedStyle:   common.DefaultPalette.Get("revisions dimmed"),
 		selectedStyle: common.DefaultPalette.Get("revisions selected"),
 	}
-	m.renderer = newRevisionListRenderer(&m, m.Sizeable)
+	m.renderer = newRevisionListRenderer(&m, m.ViewNode)
 	return &m
 }
 
@@ -709,6 +782,6 @@ func (m *Model) jumpToParent(revisions jj.SelectedRevisions) {
 	immediate, _ := m.context.RunCommandImmediate(jj.GetParent(revisions))
 	parentIndex := m.selectRevision(string(immediate))
 	if parentIndex != -1 {
-		m.cursor = parentIndex
+		m.SetCursor(parentIndex)
 	}
 }
