@@ -15,8 +15,10 @@ type EditRevSetMsg struct {
 	Clear bool
 }
 
+var _ common.Model = (*Model)(nil)
+
 type Model struct {
-	*common.Sizeable
+	*common.ViewNode
 	Editing         bool
 	autoComplete    *autocompletion.AutoCompletionInput
 	keymap          keymap
@@ -68,7 +70,7 @@ func New(context *appContext.MainContext) *Model {
 	autoComplete.Focus()
 
 	return &Model{
-		Sizeable:        &common.Sizeable{Width: 0, Height: 0},
+		ViewNode:        &common.ViewNode{Width: 0, Height: 0},
 		context:         context,
 		Editing:         false,
 		keymap:          keymap{},
@@ -81,6 +83,10 @@ func New(context *appContext.MainContext) *Model {
 }
 
 func (m *Model) Init() tea.Cmd {
+	// Ensure CurrentRevset is initialized
+	if m.context.CurrentRevset == "" {
+		m.context.CurrentRevset = m.context.DefaultRevset
+	}
 	return nil
 }
 
@@ -112,22 +118,25 @@ func (m *Model) SetHistory(history []string) {
 	m.historyActive = false
 }
 
-func (m *Model) Update(msg tea.Msg) (*Model, tea.Cmd) {
+func (m *Model) Update(msg tea.Msg) tea.Cmd {
 	switch msg := msg.(type) {
 	case tea.KeyMsg:
 		if !m.Editing {
-			return m, nil
+			return nil
 		}
 		switch msg.Type {
 		case tea.KeyCtrlC, tea.KeyEsc:
 			m.Editing = false
 			m.autoComplete.Blur()
-			return m, nil
+			return nil
 		case tea.KeyEnter:
 			m.Editing = false
 			m.autoComplete.Blur()
 			value := m.autoComplete.Value()
-			return m, tea.Batch(common.Close, common.UpdateRevSet(value))
+			if strings.TrimSpace(value) == "" {
+				value = m.context.DefaultRevset
+			}
+			return tea.Batch(common.Close, common.UpdateRevSet(value))
 		case tea.KeyUp:
 			if len(m.History) > 0 {
 				if !m.historyActive {
@@ -140,8 +149,11 @@ func (m *Model) Update(msg tea.Msg) (*Model, tea.Cmd) {
 					m.autoComplete.SetValue(m.History[m.historyIndex])
 					m.autoComplete.CursorEnd()
 				}
-				return m, nil
+			} else {
+				m.autoComplete.SetValue(m.context.CurrentRevset)
 			}
+
+			return nil
 		case tea.KeyDown:
 			if m.historyActive {
 				if m.historyIndex > 0 {
@@ -153,7 +165,7 @@ func (m *Model) Update(msg tea.Msg) (*Model, tea.Cmd) {
 					m.autoComplete.SetValue(m.currentInput)
 				}
 				m.autoComplete.CursorEnd()
-				return m, nil
+				return nil
 			}
 		}
 	case common.UpdateRevSetMsg:
@@ -168,12 +180,10 @@ func (m *Model) Update(msg tea.Msg) (*Model, tea.Cmd) {
 		}
 		m.historyActive = false
 		m.historyIndex = -1
-		return m, m.autoComplete.Init()
+		return m.autoComplete.Init()
 	}
 
-	var cmd tea.Cmd
-	m.autoComplete, cmd = m.autoComplete.Update(msg)
-	return m, cmd
+	return m.autoComplete.Update(msg)
 }
 
 func (m *Model) View() string {
@@ -182,11 +192,7 @@ func (m *Model) View() string {
 	if m.Editing {
 		w.WriteString(m.autoComplete.View())
 	} else {
-		revset := m.context.DefaultRevset
-		if m.context.CurrentRevset != "" {
-			revset = m.context.CurrentRevset
-		}
-		w.WriteString(m.styles.textStyle.Render(revset))
+		w.WriteString(m.styles.textStyle.Render(m.context.CurrentRevset))
 	}
 	return lipgloss.Place(m.Width, m.Height, 0, 0, w.String(), lipgloss.WithWhitespaceBackground(m.styles.textStyle.GetBackground()))
 }
